import os

APP_NAME = os.environ['APP_NAME']

DATABASE_HOST = os.environ['DATABASE_HOST']
DATABASE_PASS = os.environ['DATABASE_PASS']
DATABASE_PORT = os.environ['DATABASE_PORT']
DATABASE_NAME = os.environ['DATABASE_NAME']
DATABASE_USER = os.environ['DATABASE_USER']


from src.settings import DATABASE_HOST
from src.settings import DATABASE_PASS
from src.settings import DATABASE_PORT
from src.settings import DATABASE_NAME
from src.settings import DATABASE_USER

import sqlalchemy


def create_pool(database_conn_url):
    # Creating database connection pool
    db_pool = sqlalchemy.create_engine(
        database_conn_url,
        pool_size=5,
        max_overflow=2,
        pool_timeout=30,
        pool_recycle=1800
    )
    return db_pool


database_conn_url = f'postgresql+pg8000://{DATABASE_USER}:{DATABASE_PASS}@{DATABASE_HOST}:{DATABASE_PORT}/{DATABASE_NAME}'
db_pool = create_pool(database_conn_url)



import os
from pathlib import Path

from write import write

from tests.util.json_util import JsonUtil
from tests.util import backup_util
from tests.util.enum_http_method import HTTPMethod
from tests.util.requests_util import RequestsUtil
from tests.util.tcase_util import TCaseUtil


def delete_all_files_in_folder(folder: Path):
    for file in folder.iterdir():
        file.unlink()


class TCaseTools:
    @staticmethod
    def criar_caso_teste_padrao(tenant, endpoint: str, mope_code: str, http_method: HTTPMethod,
                                status_http_esperado: int, nome_caso_teste: str, 
                                dict_entrada_ou_parametros_get, executar_e_gerar_saida: bool,
                                sobrescrever_artefatos: bool = False, sobrescrever_script: bool = False,
                                utilizar_sql_global: bool = False):
        os.environ["TESTS_TENANT"] = str(tenant)
        port = os.getenv('SERVER_PORT', 5000)
        nome_arq = f"{nome_caso_teste}_{status_http_esperado}"
        nome_arq_script = f"_{http_method.name.lower()}"
        pasta_atual = __file__
        pasta_atual = Path(pasta_atual)
        pasta_util = pasta_atual.parent
        pasta_testes = pasta_util.parent
        pasta_api = Path(str(pasta_testes) + "/api")
        pasta_casos = Path(str(pasta_api) + "/casos_de_teste")
        _endpoint = endpoint
        if _endpoint[0] == '/':
            _endpoint = _endpoint[1:]
        sub_endpoints = _endpoint.split('/')
        pasta_endpoint = pasta_casos
        for sub in sub_endpoints:
            pasta_endpoint = Path(str(pasta_endpoint) + f"/{sub}")
            pasta_endpoint.mkdir(exist_ok=True)
        pasta_verbo = Path(str(pasta_endpoint) + f"/{http_method.value.lower()}")
        pasta_verbo.mkdir(exist_ok=True)
        pasta_csv_geral = Path(str(pasta_verbo) + "/dump_csv")
        pasta_csv = Path(str(pasta_csv_geral) + f"/{nome_arq}")
        pasta_sql = Path(str(pasta_verbo) + "/dump_sql")
        pasta_entradas = Path(str(pasta_verbo) + "/entradas_json")
        pasta_saidas = Path(str(pasta_verbo) + "/saidas_json")
        pasta_csv_geral.mkdir(exist_ok=True)
        pasta_csv.mkdir(exist_ok=True)
        pasta_sql.mkdir(exist_ok=True)
        pasta_entradas.mkdir(exist_ok=True)
        pasta_saidas.mkdir(exist_ok=True)
        script = Path(str(pasta_verbo) + f"/test_{sub.lower().replace('-', '_')}_{http_method.name.lower()}.py")

        arq_entrada = Path(str(pasta_entradas) + f"/{nome_arq}.json")
        arq_saida = Path(str(pasta_saidas) + f"/{nome_arq}.json")

        if arq_entrada.exists() and not sobrescrever_artefatos:
            return

        print("Criando arquivo de entrada")
        if arq_entrada.exists():
            arq_entrada.unlink()
        if arq_saida.exists():
            arq_saida.unlink()
        write(arq_entrada, JsonUtil().encode(dict_entrada_ou_parametros_get))

        print("Criando Dump em CSV")
        if pasta_csv.exists():
            delete_all_files_in_folder(pasta_csv)

        backup_util.BackupUtil.backup_database_to_csv(str(pasta_csv), tenant)


        print("#Criar script de teste")
        script_template = Path(str(pasta_util) + "/resources/caso_teste_template.py.template")
        with open(script_template, "r") as f:
            template = f.read()
        template = template.replace("<sub_endpoint>", "".join(map(str.capitalize, sub.split("-"))))
        template = template.replace("<endpoint>", endpoint)
        template = template.replace("<http_method>", http_method.value)
        template = template.replace("<method_lower>", http_method.value.lower())
        template = template.replace("<mope_code>", mope_code)
        template = template.replace("<use_global_sql_file>", str(utilizar_sql_global))
        if script.exists():
            if sobrescrever_script:
                script.unlink()
                write(str(script), template)
        else:
            write(str(script), template)
        
        # Gerar SQLs que limpam o banco
        if not utilizar_sql_global:
            texto_sql = ""
            for arquivo in pasta_csv.iterdir():
                nome = arquivo.name.replace(".csv", "")
                texto_sql += f" \r\ndelete from {nome} where tenant=:tenant;";
            arquivo_sql = Path(str(pasta_sql) + f"/{nome_arq}.sql")
            arquivo_sql_after = Path(str(pasta_sql) + f"/{nome_arq}_after.sql")
            write(str(arquivo_sql), texto_sql)
            write(str(arquivo_sql_after), texto_sql)
            

        if executar_e_gerar_saida:
            test_util = TCaseUtil(None, mope_code, endpoint, port)
            if HTTPMethod.POST == http_method:
                saida = RequestsUtil.post(test_util.endpoint, data=dict_entrada_ou_parametros_get)
            elif HTTPMethod.GET == http_method:
                saida = RequestsUtil.get(test_util.endpoint, params=dict_entrada_ou_parametros_get)
            elif HTTPMethod.PUT == http_method:
                saida = RequestsUtil.put(test_util.endpoint, data=dict_entrada_ou_parametros_get)
            elif HTTPMethod.DELETE == http_method:
                saida = RequestsUtil.delete(test_util.endpoint, params=dict_entrada_ou_parametros_get)

            print(saida.content)
            write(str(arq_saida), saida.content)


if __name__ == '__main__':
    TCaseTools.criar_caso_teste_padrao(
        1,
        "recursos/c54cca2c-1468-4eb1-88a0-b16d891985bb",
        "1234",
        HTTPMethod.DELETE,
        204,
        "exemplo1",
        JsonUtil().decode(""""""),
        True,
        True
    )


import re


class AssertUtil:

    @staticmethod
    def assert_status_code(response, expected_status_code):
        assert response.status_code == expected_status_code, \
            f"Expected status {expected_status_code}, but received {response.status_code}"

    @staticmethod
    def assert_is_instance(content, expected_type): 
        assert isinstance(content, expected_type), f"Object is not of type {expected_type}"

    @staticmethod
    def assert_is_dict(content):
        AssertUtil.assert_is_instance(content, dict)

    @staticmethod
    def assert_is_list(content):
        AssertUtil.assert_is_instance(content, list)

    @staticmethod
    def assert_same_type(a, b):
        assert type(a) is type(b), f"Objects are of type {type(a)} and {type(b)}, respectively"

    @staticmethod
    def assert_empty(collection, message = None):
        if not message:
            message = "List is not empty"
        assert len(collection) == 0, message

    @staticmethod
    def assert_dict_has_key(key, content: dict, dict_name: str = ""):
        AssertUtil.assert_is_instance(content, dict)
        assert key in content, f"Key '{key}' is not in dictionary {dict_name}"

    @staticmethod
    def assert_is_equal(a, b, message = None):
        if not message:
            message = f"{a} != {b}"
        assert a == b, message 

    @staticmethod
    def assert_is_not_equal(a, b, message = None):
        if not message:
            message = f"{a} == {b}"
        assert a != b, message 

    @staticmethod
    def assert_greater_than(a, b, message = None):
        if not message:
            message = f"{a} <= {b}"
        assert a > b, message

    @staticmethod
    def assert_less_than(a, b, message = None):
        if not message:
            message = f"{a} >= {b}"
        assert a < b, message
    
    @staticmethod
    def assert_greater_than_or_equal_to(a, b, message = None):
        if not message:
            message = f"{a} < {b}"
        assert a >= b, message

    @staticmethod
    def assert_less_than_or_equal_to(a, b, message = None):
        if not message:
            message = f"{a} > {b}"
        assert a <= b, message

    @staticmethod
    def assert_regex(content, regex):
        assert re.search(regex, content)

    @staticmethod
    def assert_statement(statement, message):
        assert statement, message



import csv
from datetime import datetime
import json
import pathlib
from typing import List, Tuple, Union

from tests.src.factory.util_factory import TestesFactory


class DumpUtil():

    @staticmethod
    def dump_from_sqls(files_paths: List[str], params: Union[List[dict], dict]):
        if isinstance(params, dict):
            params = [params for _ in range(len(files_paths))]

        for file_path, file_params in zip(files_paths, params):
            DumpUtil.dump_sql_if_exists(file_path, file_params)

    @staticmethod
    def dump_csvs_from_folder(folder_path: str, params: Union[List[dict], dict]):

        files = DumpUtil.get_files_from_folder(folder_path)

        if isinstance(params, dict):
            params = [params for _ in range(len(files))]

        for file_path, file_params in zip(files, params):
            file_name = DumpUtil.get_file_name(file_path)
            DumpUtil.dump_csv_if_exists(
                file_path, table=file_name, params=file_params)

    @staticmethod
    def dump_sql_command(sql_command: str, params: dict):
        repository = TestesFactory.getTestesRepository()
        response = repository.execute(sql_command, params)
        return response

    @staticmethod
    def fetchAll(sql_command: str, params: dict):
        repository = TestesFactory.getTestesRepository()
        response = repository.fetchAll(sql_command, params)
        return response

    @staticmethod
    def fetchOne(sql_command: str, params: dict):
        repository = TestesFactory.getTestesRepository()
        response = repository.fetchOne(sql_command, params)
        return response

    @staticmethod
    def select_from(table: str, 
                    where: Union[str, List[str]] = "", 
                    fields: Union[str, List[str]] = "*",
                    group_by: Union[str, List[str]] = "",
                    having: Union[str, List[str]] = "",
                    fetchOne: bool = False) -> List:

        if fields and isinstance(fields, list):
            fields = ", ".join([field for field in fields])
        
        if where:
            if isinstance(where, list):
                where = "WHERE " + " and ".join(where)
            elif isinstance(where, str):
                where = "WHERE " + where
        
        if group_by:
            if isinstance(group_by, list):
                group_by = "GROUP BY " + ", ".join([field for field in group_by])
            elif isinstance(group_by, str):
                group_by = f"GROUP BY {group_by}"

        if having:
            if isinstance(having, list):
                having = "HAVING " + " and ".join(having)
            elif isinstance(having, str):
                having = "HAVING " + having

        sql_command = f"SELECT {fields} FROM {table} {where} {group_by} {having};"

        repository = TestesFactory.getTestesRepository()

        if fetchOne:
            return repository.fetchOne(sql_command, {})
        return repository.fetchAll(sql_command, {})

    @staticmethod
    def count_rows(table: str, field: str = "*", where: Union[str, List[str]] = "") -> int:

        select = f"COUNT({field}) as n_rows"

        res = DumpUtil.select_from(table, where=where, fields=select, fetchOne=True)

        return res["n_rows"]

    @staticmethod
    def dump_from_sql(file_path: str, params: dict = {}):

        with open(file_path, "r", encoding='utf8') as sql_file:
            sql_command = sql_file.read()
            DumpUtil.dump_sql_command(sql_command, params)

    @staticmethod
    def dump_from_csv(file_path: str, table: str, params: dict = {}, delimiter: str = ','):

        with open(file_path, 'r', encoding='utf8') as csv_file:

            csv_reader = csv.DictReader(csv_file, delimiter=delimiter)

            columns = csv_reader.fieldnames

            table_columns = (', ').join(columns)
            table_values = (', ').join([f":{col}" for col in columns])

            sql_command = f"INSERT INTO {table} ({table_columns}) VALUES ({table_values});"

            def return_none_if_empty_string(
                x): return None if x == "" or x == None else x

            values = [{key: params[key] if key in params else return_none_if_empty_string(val) for key, val in row.items()}
                      for row in csv_reader]

            repository = TestesFactory.getTestesRepository()
            repository.execute_many(sql_command, values)

    @staticmethod
    def dump_sql_if_exists(file_path, params: dict = {}):
        file_path = file_path if file_path[-4:
                                           ] == ".sql" else f"{file_path}.sql"

        if pathlib.Path(file_path).is_file():
            DumpUtil.dump_from_sql(file_path, params)

    @staticmethod
    def dump_csv_if_exists(file_path: str, table: str, params: dict = {}):
        file_path = file_path if file_path[-4:
                                           ] == ".csv" else f"{file_path}.csv"
        if pathlib.Path(f"{file_path}").is_file():
            DumpUtil.dump_from_csv(file_path, table, params)

    @staticmethod
    def load_json(file_path: str) -> dict:
        try:
            with open(file_path, encoding='utf8') as json_file:
                return json.load(json_file)
        except: # arquivo vazio ou json incorreto
            return ''

    @staticmethod
    def write_json(file_path: str, data: dict):
        folder = DumpUtil.get_folder_path(file_path)
        # pathlib.Path(folder).mkdir(parents=False, exist_ok=True)
        DumpUtil.mkdir(folder, False, True)
        with open(file_path, 'w', encoding='utf8') as json_file:
            json_file.write(json.dumps(data, indent=2))

    @staticmethod
    def read_file_to_list(file_path: str) -> List:
        itens = []
        with open(file_path, "r", encoding='utf8') as file:
            for line in file:
                itens.append(line.strip())
        return itens

    @staticmethod
    def write_list_to_file(file_path: str, content_list: List):
        with open(file_path, "w", encoding='utf8') as file:
            for item in content_list:
                file.write(f"{item}\n")

    @staticmethod
    def mkdir(folder_path: str, parents: bool = False, exist_ok: bool = True):
        pathlib.Path(folder_path).mkdir(parents=parents, exist_ok=exist_ok)

    @staticmethod
    def load_json_data_and_status_code(file_path: str) -> Tuple[dict, int]:
        expected_status = int(DumpUtil.get_file_name(file_path).split("_")[-1])
        data = DumpUtil.load_json(file_path)
        return data, expected_status

    @staticmethod
    def get_status_code_from_file_path(file_path):
        return int(DumpUtil.get_file_name(file_path).split("_")[-1])

    @staticmethod
    def is_file(file_path: str):
        return pathlib.Path(file_path).is_file()

    @staticmethod
    def copy_file(source_file_path: str, dest_file_path: str):
        with open(source_file_path, "r", encoding='utf8') as source_file, open(dest_file_path, "w", encoding='utf8') as dest_file:
            for line in source_file:
                dest_file.write(line)

    @staticmethod
    def get_file_name(file_path: str) -> str:
        return str(pathlib.Path(file_path).stem)

    @staticmethod
    def get_file_name_with_extension(file_path: str) -> str:
        return str(pathlib.Path(file_path).name)

    @staticmethod
    def get_folder_path(file_path: str) -> str:
        return str(pathlib.Path(file_path).parent.resolve())

    @staticmethod
    def get_workspace_path() -> str:
        return str(pathlib.Path().resolve())

    @staticmethod
    def get_tests_folder_path() -> str:
        return str(pathlib.Path(DumpUtil.get_workspace_path(), "tests/api/casos_de_teste"))

    @staticmethod
    def get_dump_sql_folder_path() -> str:
        return str(pathlib.Path(DumpUtil.get_tests_folder_path(), "dump_sql"))

    @staticmethod
    def get_files_from_folder(folder_path: str, return_name: bool = False) -> Union[List, List[Tuple[str, str]]]:

        if pathlib.Path(folder_path).exists():
            files = pathlib.Path(f"{folder_path}").iterdir()

            if not return_name:
                return list(map(str, files))

            return [(DumpUtil.get_file_name(file_path), str(file_path)) for file_path in files]
        else:
            return []


import os
from uuid import uuid4

from tests.util.json_util import JsonUtil


class GlobalUtil:
    __request = None

    @staticmethod
    def profile_query(datetime_inicio, datetime_fim, sql, parametros, retorno):
        request = GlobalUtil.get_request()
        try:
            request.parametros["tenant"]
        except:
            pass
        if os.getenv("USA_PROFILE", "FALSE").lower() == "true":

            if isinstance(request, RequestMock):
                return

            rastro = {
                "id": uuid4(),
                "tenant": request.parametros["tenant"],
                "id_requisicao": request.parametros["id"],
                "datahora_inicio": datetime_inicio,
                "datahora_fim": datetime_fim,
                "string_sql": sql,
                "parametros": JsonUtil().encode(parametros),
                "qnt_linhas_parametros": len(parametros) if isinstance(parametros, list) else 1,
                "qnt_campos_parametros": JsonUtil().encode(parametros).count(':') if parametros is not None else 0,
                "qnt_linhas_retorno": len(retorno) if retorno is not None else 0
            }
            if not "rastros" in request.parametros:
                request.parametros["rastros"] = []
            request.parametros["rastros"].append(rastro)

    @staticmethod
    def get_request() -> any:
        try:
            from flask import request
            x = request.args

            try:
                y = request.parametros
            except:
                request.parametros = {}

            return request
        except:
            if GlobalUtil.__request is None:
                GlobalUtil.__request = RequestMock()
                GlobalUtil.__request
            return GlobalUtil.__request


class RequestMock:
    parametros = {}



class CursorUtil:
    @staticmethod
    def fetchall(cursor):
        fetch =  cursor.fetchall()
        retorno = []
        for f in fetch:
            linha ={}
            for key in f.keys():
                linha[key] = f[key]
            retorno.append(linha)
        return retorno

    @staticmethod
    def fetchone(cursor):
        if cursor is not None:
            dados = cursor.fetchall()
            if len(dados) > 0:
                return dados[0]


from typing import Tuple, Union

from tests.util.assert_util import AssertUtil
from tests.util.dump_util import DumpUtil


class ValidateUtil():

    @staticmethod
    def compare_content(a, b, diff_path: str = None) -> Tuple[bool, Union[dict, list, None]]:

        if not type(b) is type(a):
            return True, None

        has_diff = False
        diff = {}

        if isinstance(a, dict):
            diff = ValidateUtil.compare_dict(a, b)
            has_diff = diff != {}

        elif isinstance(a, list):
            diff = ValidateUtil.compare_list(a, b)
            has_diff = len(diff) != 0

        elif a != b:
            has_diff = True

        if has_diff and diff_path:
            DumpUtil.write_json(diff_path, diff)

        return has_diff, diff

    @staticmethod
    def compare_dict(dict_a: dict, dict_b: dict) -> dict:

        diff = {}

        for key, val_a in dict_a.items():

            if key not in dict_b:
                diff[key] = "Missing property"
                continue

            val_b = dict_b[key]

            if not type(val_b) is type(val_a):
                diff[key] = f"Incorrect type: {type(val_a)} != {type(val_b)}"
                continue

            val_diff = {}
            if isinstance(val_a, dict):
                val_diff = ValidateUtil.compare_dict(val_a, val_b)

            elif isinstance(val_a, list):
                val_diff = ValidateUtil.compare_list(val_a, val_b)

            elif val_a != val_b:
                val_diff = f"Different value -> {val_a} != {val_b}"

            if val_diff:
                diff[key] = val_diff

        return diff

    @staticmethod
    def compare_list(list_a: list, list_b: list) -> list:

        if len(list_a) > len(list_b):
            return "Different lenght"

        diff = []

        for i, (val_a, val_b) in enumerate(zip(list_a, list_b)):

            if not type(val_b) is type(val_a):
                diff.append(f"Incorrect type {type(val_a)} != {type(val_b)}")
                continue

            val_diff = {}
            if isinstance(val_a, dict):
                val_diff = ValidateUtil.compare_dict(val_a, val_b)

            elif isinstance(val_a, list):
                val_diff = ValidateUtil.compare_list(val_a, val_b)

            elif val_a != val_b:
                val_diff = f"Different value at {i} -> {val_a} != {val_b}"

            if val_diff:
                val_diff["index"] = i
                diff.append(val_diff)

        return diff

    # -----------------------------------------------------------   
    @staticmethod
    def assert_content(esperado, real, root="json"):
        AssertUtil.assert_same_type(esperado, real)

        failures = []
        if isinstance(real, dict):
            failures = ValidateUtil.assert_dict(esperado, real, trace=root)
        elif isinstance(real, list):
            failures = ValidateUtil.assert_list(esperado, real, trace=root)
        else:
            if real != esperado:
                failures = [f"\n\t{root}:  real [{real}]  != esperado [{esperado}]"]

        AssertUtil.assert_statement(len(failures) == 0, "\n".join([f"{fail}" for fail in failures]))

    @staticmethod
    def assert_dict(esperado: dict, real: dict, trace: str = ""):

        failures = []
        for key, val_a in esperado.items():

            trace_key = f"{trace} -> {key}"

            if key not in real:
                failures.append(f"\n\t{trace_key} is not in dictionary")
                continue

            val_b = real[key]

            if not type(val_a) is type(val_b):
                failures.append(f"\n\t{trace_key}: Incorrect type: {type(val_a)} != {type(val_b)}")
                continue

            key_failures = []

            if isinstance(val_a, dict):
                key_failures = ValidateUtil.assert_dict(val_a, val_b, trace_key)

            elif isinstance(val_b, list):
                key_failures = ValidateUtil.assert_list(val_a, val_b, trace_key)

            else:
                if (val_a != val_b) and (key != 'tenant'):
                    key_failures = [f"\n\t{trace_key}:  {val_a} != {val_b}"]

            if key_failures:
                failures.extend(key_failures)

        return failures

    @staticmethod
    def assert_list(esperado: list, real: list, trace: str = ""):

        if len(esperado) != len(real):
            return [f"\n\t{trace}: Lists do not have same length: Real [{len(real)}] != Esperado [{len(esperado)}]"]

        failures = []

        for i, (val_a, val_b) in enumerate(zip(esperado, real)):

            trace_index = f"{trace}[{i}]"

            index_failures = None

            if isinstance(val_a, dict):
                index_failures = ValidateUtil.assert_dict(val_a, val_b, trace_index)

            elif isinstance(val_b, list):
                index_failures = ValidateUtil.assert_list(val_a, val_b, trace_index)

            else:
                if val_a != val_b:
                    index_failures = [f"\n\t{trace_index}:  {val_a} != {val_b}"]

            if index_failures:
                failures.extend(index_failures)

        return failures



import os

import requests

from tests.util.enum_param_mode import ParamMode

api_key = os.getenv("TESTS_API_KEY", "")
content_type = 'application/json'


class RequestsUtil():

    @staticmethod
    def get_headers_with_api_key(headers):
        if headers:
            headers["X-API-Key"] = headers["X-API-Key"] if headers["X-API-Key"] else api_key
        else:
            headers = {"X-API-Key": api_key}
        return headers

    @staticmethod
    def get(url, data=None, params: dict = None, headers: dict = None,
            param_mode: ParamMode = ParamMode.QUERY) -> requests.Response:

        headers = RequestsUtil.get_headers_with_api_key(headers)
        headers['content-type'] = 'application/json'

        if param_mode == ParamMode.PATH:
            for field, value in params.items():
                url = url.replace(f"<{field}>", f"{value}")
            params = {}

        return requests.get(url=url, json=data,
                            params=params, headers=headers)

    @staticmethod
    def delete(url, data=None, params: dict = None, headers: dict = None,
            param_mode: ParamMode = ParamMode.QUERY) -> requests.Response:

        headers = RequestsUtil.get_headers_with_api_key(headers)
        headers['content-type'] = 'application/json'

        if param_mode == ParamMode.PATH:
            for field, value in params.items():
                url = url.replace(f"<{field}>", f"{value}")
            params = {}

        return requests.delete(url=url, json=data,
                            params=params, headers=headers)

    @staticmethod
    def post(url, data: dict = None, params: dict = None, headers: dict = None) -> requests.Response:

        headers = RequestsUtil.get_headers_with_api_key(headers)
        headers['content-type'] = 'application/json'

        retorno = requests.post(
            url=url, json=data, headers=headers, params=params)
        return retorno


    @staticmethod
    def put(url, data: dict = None, params: dict = None, headers: dict = None) -> requests.Response:

        headers = RequestsUtil.get_headers_with_api_key(headers)
        headers['content-type'] = 'application/json'

        retorno = requests.put(
            url=url, json=data, headers=headers, params=params)
        return retorno



import decimal
import enum
import json
from datetime import date, datetime
from uuid import UUID

import jsonpickle

from pydantic import BaseModel


class JsonUtil:
    def encode(self, dados: object, __profundidade_permitida=20):
        if isinstance(dados, str):
            return dados
        if dados is None:
            return None
        if not isinstance(dados, dict):
            dados = self.toDict(dados, __profundidade_permitida)
        return json.dumps(dados, indent=True, sort_keys=True, default=self.toDict)

    def decode(self, dados: object):
        if dados is None:
            return None
        if dados == '':
            return None
        return jsonpickle.decode(dados)

    def toDict(self, dados: object, __profundidade_permitida=10):
        """
        Converte um Objeto em um dicionario. Caso seja um tipo primitivo, retorna o proprio argumento recebido
        """
        if __profundidade_permitida <= 0:
            return str(dados)
        if (isinstance(dados, BaseModel)):
            return self.toDict(dados.dict(), __profundidade_permitida)
        if (isinstance(dados, dict)):  # A entrada já foi um dict
            saida = dict()
            for key in dados.keys():
                saida[key] = self.toDict(dados[key], __profundidade_permitida - 1)
            return saida
        if (isinstance(dados, UUID)):
            return str(dados)
        if (isinstance(dados, list)):
            saida = list()
            for elem in dados:
                saida.append(self.toDict(elem, __profundidade_permitida - 1))
            return saida
        if isinstance(dados, enum.Enum):
            return dados.value
        if (isinstance(dados, datetime)):  # Tratamento de datas
            return dados.strftime('%Y-%m-%d %H:%M:%S')
        if (isinstance(dados, date)):  # Tratamento de datas
            return dados.strftime('%Y-%m-%d')
        if isinstance(dados, decimal.Decimal):  # Tratamento de decimal
            return float(dados)
        if (not hasattr(dados, "__dict__")):  # A entrada nao pode ser convertida para dict
            return dados
        else:
            saida = dict()  # A entrada foi um objeto, que deverá ser convertido para dict
            # Para cada atributo...
            dir_dados = dir(dados)
            for atributo in dir_dados:
                if atributo.startswith('__'):
                    continue
                try:
                    if callable(getattr(dados, atributo)):
                        continue
                except:
                    continue
                if atributo.startswith('_'):
                    continue
                atr = getattr(dados, atributo)
                saida[atributo] = self.toDict(atr, __profundidade_permitida - 1)
            return saida

    def encode_soft(self, dados, __profundidade_permitida=10):
        if isinstance(dados, str):
            return dados
        if dados is None:
            return None
        dic = self.toDict(dados, __profundidade_permitida)
        for key in dic.keys():
            dic[key] = str(dic[key])
        return json.dumps(dic)



import os

from tests.src.factory.util_factory import TestesFactory
from tests.util.assert_util import AssertUtil
from tests.util.dump_util import DumpUtil
from tests.util.enum_http_method import HTTPMethod
from tests.util.enum_param_mode import ParamMode
from tests.util.requests_util import RequestsUtil
from tests.util.validate_util import ValidateUtil


class TCaseUtil:
    def __init__(self, current_file, mope_code, endpoint, port = 5000):
        """
        Args:
            current_file_path: caminho do arquivo local de teste. Normalmente passado com __file__
        """
        self.current_file = current_file
        self.dump_sql_folder_path = DumpUtil.get_dump_sql_folder_path()
        if current_file is not None:
            self.folder_path = DumpUtil.get_folder_path(current_file)
            self.argvalues = DumpUtil.get_files_from_folder(

                f"{self.folder_path}/entradas_json", True)
        self.url_base = os.getenv("TESTS_URL_BASE", "http://localhost")
        self.tests_tenant = os.getenv("TESTS_TENANT", "11045")
        self.mope_code = mope_code

        self.global_sql = f"{self.dump_sql_folder_path}/global.sql"
        self.tests_tenant = os.getenv("TESTS_TENANT", "100011045")
        self.endpoint = f"{self.url_base}:{port}/{self.mope_code}/{endpoint}"

    def pre_setup(self, json_entrada_nome, executar_globals=True):
        print(f"\nJSON: {json_entrada_nome}", end="\t")
        entrada_sql = f"{self.folder_path}/dump_sql/{json_entrada_nome}"
        csv_folder = f"{self.folder_path}/dump_csv/{json_entrada_nome}"

        params_tenant = {"tenant": self.tests_tenant}
        if executar_globals:
            DumpUtil.dump_from_sqls([self.global_sql], params_tenant)
        DumpUtil.dump_from_sqls([entrada_sql], params_tenant)
        DumpUtil.dump_csvs_from_folder(

            csv_folder, {"tenant": self.tests_tenant})

    def pos_setup(self, json_entrada_nome=None):
        params_tenant = {"tenant": self.tests_tenant}
        DumpUtil.dump_sql_if_exists(
            f"{self.dump_sql_folder_path}/global_after", params_tenant)
        if json_entrada_nome is not None:
            DumpUtil.dump_sql_if_exists(
                f"{self.folder_path}/dump_sql/{json_entrada_nome}_after", params_tenant)

    def common_request_test(self, json_entrada, json_entrada_nome, http_method, param_mode=ParamMode.QUERY):

        body, status_esperado = DumpUtil.load_json_data_and_status_code(
            json_entrada)
        if HTTPMethod.POST == http_method:
            retorno = RequestsUtil.post(self.endpoint, data=body)
        elif HTTPMethod.GET == http_method:
            retorno = RequestsUtil.get(
                self.endpoint, params=body, param_mode=param_mode)
        elif HTTPMethod.DELETE == http_method:
            retorno = RequestsUtil.delete(
                self.endpoint, params=body, param_mode=param_mode)
        elif HTTPMethod.PUT == http_method:
            retorno = RequestsUtil.put(self.endpoint, data=body)
        else:
            retorno = None
        AssertUtil.assert_status_code(retorno, status_esperado)
        content = ''
        if retorno.text.strip() != '':
            content: str = retorno.json()
        saida = DumpUtil.load_json(
            f"{self.folder_path}/saidas_json/{json_entrada_nome}.json")
        ValidateUtil.assert_content(saida, content)

    def validar_banco(self):
        pass



import csv
import json
import os
import pathlib
from typing import List

import requests

from tests.src.repository.testes_repository import TestesRepository
from tests.src.factory.util_factory import TestesFactory
from tests.util.dump_util import DumpUtil

url_base = os.getenv("TESTS_URL_BASE", "http://localhost")
tests_tenant = os.getenv("TESTS_TENANT", "11045")

folder_path = DumpUtil.get_folder_path(__file__)


class BackupUtil:

    @staticmethod
    def backup_table_to_csv(dest_file_path: str, table: str, tenant: int,
                            repository: TestesRepository = TestesFactory.getTestesRepository()):

        sql = f"SELECT * FROM {table} where tenant={tenant}"

        rows = repository.fetchAll(sql, {})
        for row in rows:
            if "tenant" in row.keys():
                row["tenant"] = ":tenant"
        if not rows:
            return

        fieldnames = rows[0].keys()

        with open(dest_file_path, "w", newline="", encoding='utf8') as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames)
            writer.writeheader()
            for row in rows:
                writer.writerow(row)

    @staticmethod
    def backup_tables_to_csv(dest_folder_path: str, tenant: int, tables: List = []):

        DumpUtil.mkdir(dest_folder_path, parents=True)

        if not tables:
            return

        repository = TestesFactory.getTestesRepository()

        for table in tables:
            if table not in ['django_migrations', 'usuarios', 'registros_requisicoes', 'rastros']:
                BackupUtil.backup_table_to_csv(
                    f"{dest_folder_path}/{table}.csv", table, tenant, repository)

    # TODO Aplicar recurso para qualquer
    @staticmethod
    def backup_database_to_csv(dest_folder_path: str, tenant: int):
        repository = TestesFactory.getTestesRepository() 
        response = repository.fetchAll("select tablename from pg_catalog.pg_tables where schemaname = 'public';", {})


        ignore_tables = ["vw_timeserver", "registros_requisicoes"]

        tables = [val['tablename']for val in response  if val["tablename"] not in ignore_tables]

        BackupUtil.backup_tables_to_csv(dest_folder_path, tenant, tables)

    @staticmethod
    def get_csvs_changes(csv_a_path: str, csv_b_path: str, csv_c_path: str):

        csv_a_rows = DumpUtil.read_file_to_list(csv_a_path)
        csv_b_rows = DumpUtil.read_file_to_list(csv_b_path)

        fieldnames = csv_a_rows[0]

        csv_a_rows = csv_a_rows[1:]
        csv_b_rows = csv_b_rows[1:]

        csv_c_rows = [row for row in csv_a_rows if row not in csv_b_rows]
        csv_c_rows.insert(0, fieldnames)

        if len(csv_c_rows) > 1:
            DumpUtil.write_list_to_file(csv_c_path, csv_c_rows)

    @staticmethod
    def get_csvs_changes_from_folder(folder_a_path: str, folder_b_path: str, folder_c_path: str):

        DumpUtil.mkdir(folder_c_path, parents=True)

        files_a_paths = DumpUtil.get_files_from_folder(folder_a_path)

        for csv_a_path in files_a_paths:
            csv_a_name = DumpUtil.get_file_name_with_extension(csv_a_path)

            csv_b_path = f"{folder_b_path}/{csv_a_name}"
            csv_c_path = f"{folder_c_path}/{csv_a_name}"

            if not DumpUtil.is_file(csv_b_path):
                DumpUtil.copy_file(csv_a_path, csv_c_path)

            else:
                BackupUtil.get_csvs_changes(csv_a_path, csv_b_path, csv_c_path)

    @staticmethod
    def delete_backup(backup_folder: str):
        files = DumpUtil.get_files_from_folder(backup_folder)

        for file_path in files:
            pathlib.Path(file_path).unlink()

        pathlib.Path(backup_folder).rmdir()

if __name__ == "__main__":
    # BackupUtil.backup_database_to_csv("/backup/antes")
    BackupUtil.backup_database_to_csv("/backup/depois", 1011045)
    BackupUtil.get_csvs_changes_from_folder(
        "/backup/depois",
        "/backup/antes",
        "/backup/alteracoes")